/* 	$Id$	 */

/* Copyright (c) 2007 Arnaud Legrand, Bruno Donnassolo.
   All rights reserved.                                          */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#include "private.h"
#include "xbt/sysdep.h"
#include "xbt/log.h"
XBT_LOG_NEW_DEFAULT_SUBCATEGORY(simix_environment, simix,
				"Logging specific to SIMIX (environment)");
/** \defgroup msg_easier_life      Platform and Application management
 *  \brief This section describes functions to manage the platform creation
 *  and the application deployment. You should also have a look at 
 *  \ref MSG_examples  to have an overview of their usage.
 *    \htmlonly <!-- DOXYGEN_NAVBAR_LABEL="Platforms and Applications" --> \endhtmlonly
 * 
 */

/********************************* SIMIX **************************************/



/** \ingroup msg_easier_life
 * \brief A platform constructor.
 *
 * Creates a new platform, including hosts, links and the
 * routing_table. 
 * \param file a filename of a xml description of a platform. This file 
 * follows this DTD :
 *
 *     \include surfxml.dtd
 *
 * Here is a small example of such a platform 
 *
 *     \include small_platform.xml
 *
 * Have a look in the directory examples/msg/ to have a big example.
 */
void SIMIX_create_environment(const char *file) 
{
  xbt_dict_cursor_t cursor = NULL;
  char *name = NULL;
  void *workstation = NULL;
  char *workstation_model_name;

  simix_config_init(); /* make sure that our configuration set is created */
  surf_timer_resource_init(file);

  /* which model do you want today? */
  workstation_model_name = xbt_cfg_get_string (_simix_cfg_set, "surf_workstation_model");

  DEBUG1("Model : %s", workstation_model_name);
  if (!strcmp(workstation_model_name,"KCCFLN05")) {
    surf_workstation_resource_init_KCCFLN05(file);
  } else if (!strcmp(workstation_model_name,"KCCFLN05_proportionnal")) {
    surf_workstation_resource_init_KCCFLN05_proportionnal(file);
  } else if (!strcmp(workstation_model_name,"CLM03")) {
    surf_workstation_resource_init_CLM03(file);
  } else {
    xbt_assert0(0,"The impossible happened (once again)");
  }
  _simix_init_status = 2; /* inited; don't change settings now */

  xbt_dict_foreach(workstation_set, cursor, name, workstation) {
    __SIMIX_host_create(name, workstation, NULL);
  }

  return;
}

